<?php

declare(strict_types=1);

namespace Drupal\fire_chaser\Service;

use Composer\InstalledVersions;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * List all installed modules with versions.
 */
class ModuleListService {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The extension list service.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  protected $extensionList;

  /**
   * Constructs a new ModuleListService.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ModuleExtensionList $extension_list_module
   *   The module extension list.
   * @param \Drupal\Core\Extension\ExtensionList $extension_list
   *   The extension list service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, ModuleExtensionList $extension_list_module, ExtensionList $extension_list) {
    $this->moduleHandler = $module_handler;
    $this->moduleExtensionList = $extension_list_module;
    $this->extensionList = $extension_list;
  }

  /**
   * Lists all modules and their versions including uninstalled.
   *
   * @return array
   *   An array of installed modules and their versions.
   */
  public function listAllModules() {
    $modules = $this->moduleExtensionList->reset()->getList();
    $module_data = [];

    foreach ($modules as $module_name => $module) {
      $other = $this->moduleHandler->moduleExists($module_name);

      if ($other) {
        $module_data[$module_name] = $this->getModuleData($module_name);
      }
      else {
        $module_data[$module_name] = [
          'name' => $module_name,
          'installed' => FALSE,
        ];

      }
    }

    return $module_data;
  }

  /**
   * Lists all installed modules and their versions.
   *
   * @return array
   *   An array of installed modules and their versions.
   */
  public function listModules() {
    $modules = $this->extensionList->getAllInstalledInfo();
    $module_data = [];

    foreach ($modules as $module_name => $module_info) {
      if (isset($module_info['package']) && $module_info['package'] == 'Core') {
        continue;
      }

      if (isset($module_info['package']) && $module_info['package'] == 'Field types') {
        continue;
      }

      if (isset($module_info['project']) && $module_info['project'] == 'drupal') {
        continue;
      }

      $module_info = $this->extensionList->getExtensionInfo($module_name);
      $module_data[$module_name] = [
        'name' => $module_info['name'],
        'type' => $module_info['type'],
        'package' => $module_info['package'] ?? 'N/A',
        'core_incompatible' => $module_info['core_incompatible'] ?? 'N/A',
        'version' => $module_info['version'] ?? 'N/A',
      ];
    }

    return $module_data;
  }

  /**
   * Return list of composer packages.
   */
  public function getComposerPackages() {
    $vendors = InstalledVersions::getAllRawData();

    if (count($vendors) > 0 && isset($vendors[0]['versions'])) {
      $packages = array_filter($vendors[0]['versions'], function ($vendor) {
        return isset($vendor['version']);
      });

      if (count($packages) > 0) {
        return $packages;
      }
    }

    return $packages = [];
  }

  /**
   * Extracts module data.
   */
  public function getModuleData($module_name) {
    $module_info = $this->extensionList->getExtensionInfo($module_name);

    return [
      'name' => $module_info['name'],
      'type' => $module_info['type'],
      'package' => $module_info['package'] ?? 'N/A',
      'core_incompatible' => $module_info['core_incompatible'] ?? 'N/A',
      'version' => $module_info['version'] ?? 'N/A',
    ];
  }

}
